function [ActLev, ActFactor] = ActiveLevel(x, Fs)

% Active speech level calculation for data scaled to +/- 32000

% $Id: ActiveLevel.m,v 1.1 2018/11/22 23:53:08 pkabal Exp $

Nl = 15;		% No. threshold levels
b = 2;			% Threshold ratio

MdB = 15.9;
EFREQ = 1000;

% Set up the subsampling factor (min rate: 1000 / sec)
Nsub = max(fix(Fs / EFREQ), 1);

% Find the tabulated activity counts
j = 0:Nl-1;
c = b.^j;

an = EnvCount(Fs, x, c, Nsub);

% Mean-square value
Nx = length(x);
Ex = sum(x' * x) / Nx;

ActLev = ALevel(an, c, Ex, MdB);
ActFactor = Ex / ActLev^2;

end

% --------------------
function an = EnvCount(Fs, x, c, Nsub)
% Accumulate envelope activity counts
%
% output: an: vector of fractional activity counts
% input:  Fs: sampling rate (Hz)
%         x: input signal
%         c: threshold values (increasing values)
%         Nsub: subsampling ratio

T = 0.03;
H = 0.2;
t = Nsub/Fs;
g = exp(-t/T);
I = ceil(H/t);

Nx = length(x);
Nl = length(c);
a = zeros(1, Nl);
h = I * ones(1, Nl);

p = 0;
q = 0;
for i = 1:Nsub:Nx
  p = g * p + (1-g) * abs(x(i));
  q = g * q + (1-g) * p;
  for j = 1:Nl
    if (q >= c(j))
      a(j) = a(j) + 1;
      h(j) = 0;
    elseif (h(j) < I)
      a(j) = a(j) + 1;
      h(j) = h(j) + 1;
    else
      break;
    end
  end
end

Nxs = ceil(Nx/Nsub);
an = a / Nxs;

end

% --------------------
function ActLev = ALevel(an, c, Ex, MdB)
% Find the active speech level
%
% output: ActLev: active speech level
% input:  an: fractional activity values (non-increasing values)
%         c:  threshold levels (increasing values)
%         Ex: mean squared value of the signal
%         MdB: offset in dB

% Default return value
ActLev = 0;

Mln = MdB * log(10) / 10;       % log(m), where m = 10^(M/10)
Nl = length(an);

% Find the solution to Aln - Cln = Mln
for j = 1:Nl
  if (an(j) == 0)
    break;
  end
  Alnj = log(Ex / an(j));
  Clnj = 2 * log(c(j));        % log(c(j)^2)
  dACj = Alnj - Clnj;
  if (j == 1)
    if (dACj <= Mln)
      ActLev = exp(Alnj / 2);
      break;
    end
  elseif (dACj <= Mln)
    alpha = (Mln - dACjp) / (dACj - dACjp);
    Alno = Alnjp + alpha * (Alnj - Alnjp);
    ActLev = exp(Alno / 2);
    break;
  end
  Alnjp = Alnj;
  dACjp = dACj;
end

end
